class Colour:
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    END = '\033[0m'

    GRAY = '\033[90m'
    RED = '\033[91m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    BLUE = '\033[94m'
    PURPLE = '\033[95m'
    CYAN = '\033[96m'
    WHITE = '\033[97m'

    DARKGRAY = '\033[30m'
    DARKRED = '\033[31m'
    DARKGREEN = '\033[32m'
    DARKYELLOW = '\033[33m'
    DARKBLUE = '\033[34m'
    DARKPURPLE = '\033[35m'
    DARKCYAN = '\033[36m'
    DARKWHITE = '\033[37m'

    FULLDARKGRAY = '\033[40m'
    FULLDARKRED = '\033[41m'
    FULLDARKGREEN = '\033[42m'
    FULLDARKYELLOW = '\033[43m'
    FULLDARKBLUE = '\033[44m'
    FULLDARKPURPLE = '\033[45m'
    FULLDARKCYAN = '\033[46m'
    FULLDARKWHITE = '\033[47m'

    FULLGRAY = '\033[100m'
    FULLRED = '\033[101m'
    FULLGREEN = '\033[102m'
    FULLYELLOW = '\033[103m'
    FULLBLUE = '\033[104m'
    FULLPURPLE = '\033[105m'
    FULLCYAN = '\033[106m'
    FULLWHITE = '\033[107m'

    def print(mesage,colour=''):
        print(colour + mesage + Colour.END)



#!/usr/bin/env python3
# PYTHON_ARGCOMPLETE_OK

import sys

from . import __mod_name__, __version__

from .dependencies.dependencies import dependency_map
from .dependencies.dependency import install_dependencies
from .lib.args import Args
from .lib.meditor import Meditor
from .lib.srbColour import Colour
from .lib.util import Util

'''
run it as:
    medipack trim inp.mp4 -s 0:30 -e 1:40

    to trim
        medipack trim inp.mp4 -s 0:30 -e 1:40 -o output.mp4
    to crop
        medipack crop inp.mp4 -t 10 -l 10 -b 10 -r 20 -o output.mp4
    to change quality/size
        medipack resize inp.mp4 -q 50 -o output.mp4
        medipack resize inp.mp4
    to extract audio
        medipack extract --audio inp.mp4 -o out.mp3
    to extract video
        medipack extract --video inp.mp4 -o out.mp4

for audio output, please specify ensure mp3 in output name
    medipack extract --audio inp.mp4 -o out.mp3             (WORKS)
    medipack extract --audio inp.mp4 -o out.mp4             (WILL FAIL)
    medipack trim inp.mp3 -s 0:30 -e 1:40 -o output.mp3     (WORKS)
'''


def main():
    parser = Args.get_parser()
    if(not install_dependencies(dependency_map)):
        print('please install the required dependencies')
        sys.exit(1)

    if parser.version:
        print(__mod_name__+'=='+__version__)
        sys.exit()


    inp,out = Util.get_io(parser)
    if(parser.action == 'extract'):
        if(inp.split('.')[-1] == 'mp3"'):
            message = parser.action + ' not supported for audio inpput'
            Colour.print(message,Colour.RED)
            sys.exit(0)
        if(parser.audio):
            Meditor.extract_audio(inp,out)
        if(parser.video):
            Meditor.extract_video(inp,out)
        sys.exit(0)

    # audio
    if(out.split('.')[-1] == 'mp3"'):
        if (parser.action in ['crop','resize']):
            message = parser.action + ' not supported for mp3 output'
            Colour.print(message,Colour.RED)
            sys.exit(0)
        elif(parser.action == 'trim'):
            trimmer = Util.get_trimmer(parser,inp)
            Meditor.aaudio_cutter(inp,trimmer,out)
        else:
            Colour.print('unknow action',Colour.YELLOW)
            sys.exit(0)

    # video
    if(parser.action == 'trim'):
        trimmer = Util.get_trimmer(parser,inp)
        Meditor.video_trimmer(inp,trimmer,out)
    elif(parser.action == 'crop'):
        filters = Util.get_filters(parser)
        Meditor.video_cropper(inp,filters,out)
    elif(parser.action == 'resize'):
        resizer = Util.get_resizer(parser)
        Meditor.video_resizer(inp,resizer,out)
    else:
        Colour.print('unknow action',Colour.YELLOW)

if(__name__=="__main__"):
    main()



class Time:
    def get_time(st):
        st = st.split(':')
        if(len(st) == 1):
            return None
        if(len(st) == 2):
            h='00'
            m,s = st
        if(len(st)==3):
            h,m,s = st
        if(len(st)>3):
            return None

        if(not Time._is_int(h) or not Time._is_int(m)):
            return None
        h = int(h)
        m = int(m)
        s = int(s)
        if(m>59 or s>59):
            return None
        return str(h).zfill(2) + ':' + str(m).zfill(2) + ':' + str(s).zfill(2)

    def get_relative_time(st,et):
        # just for safety
        st = Time.get_time(st)
        et = Time.get_time(et)

        st = st.split(':')
        sh,sm,ss = st

        et = et.split(':')
        eh,em,es = et

        sh = int(sh)
        sm = int(sm)
        ss = int(ss)

        eh = int(eh)
        em = int(em)
        es = int(es)

        if(es < ss):
            if(em == 0):
                if(eh == 0): return None
                eh -= 1
                em += 60
            em -= 1
            es +=60
        s = es - ss

        if(em < sm):
            if(eh == 0):return None
            eh -= 1
            em += 60
        m = em - sm

        if(eh < sh): return None
        h = eh - sh

        if(h == 0 and m == 0 and s == 0): return None
        return str(h).zfill(2) + ':' + str(m).zfill(2) + ':' + str(s).zfill(2)


    def _is_int(x):
        try:
            x = int(x)
        except:
            return False
        return True



import os
import subprocess as sp

from .srbColour import Colour

class Meditor:
    def getLength(filename):
        result = sp.Popen(["ffprobe", filename],
            stdout = sp.PIPE, stderr = sp.STDOUT)
        arr = [x for x in result.stdout.readlines() if "Duration".encode('utf-8') in x]
        x = arr[0].decode('utf-8')
        x = x.split(' ')
        dur = x.index('Duration:') + 1
        return x[dur].split('.')[0]

    def extract_audio(inp,out):
        video_codec = " "
        audio_codec =  " "
        exec_command = 'ffmpeg -i ' + str(inp) + video_codec + audio_codec + out
        Colour.print(exec_command,Colour.GREEN)
        os.system(exec_command)

    def extract_video(inp,out):
        video_codec = " -c:v copy "
        audio_codec =  " -an "
        exec_command = 'ffmpeg -i ' + str(inp) + video_codec + audio_codec + out
        Colour.print(exec_command,Colour.GREEN)
        os.system(exec_command)

    def video_trimmer(inp,trimmer,out):
        video_codec = " -c:v copy "
        audio_codec =  " -c:a copy "
        exec_command = 'ffmpeg -i ' + str(inp) + trimmer + video_codec + audio_codec + out
        Colour.print(exec_command,Colour.GREEN)
        os.system(exec_command)

    def video_cropper(inp,filters,out):
        video_codec = "" # senseless to say 'crop video and copy video, both at same time'
        audio_codec =  " -c:a copy "
        exec_command = 'ffmpeg -i ' + str(inp) + filters + video_codec + audio_codec + out
        Colour.print(exec_command,Colour.GREEN)
        os.system(exec_command)

    def video_resizer(inp,resizer,out):
        video_codec = "" # senseless to say 'change quality video and copy video, both at same time'
        audio_codec =  " -c:a copy "
        exec_command = 'ffmpeg -i ' + str(inp) + resizer + video_codec + audio_codec + out
        Colour.print(exec_command,Colour.GREEN)
        os.system(exec_command)

    def audio_cutter(inp,trimmer,out):
        exec_command = 'ffmpeg -i ' + str(inp) + trimmer + codec + out
        Colour.print(exec_command,Colour.GREEN)
        os.system(exec_command)




import os
import sys

try:
    import argparse
    from argcomplete import autocomplete
except:
    err = """
    You haven't installed the required dependencies.
    """
    print(err)
    sys.exit(0)

from .srbColour import Colour


class Args:
    def validate_crop_args(parser):
        if(not parser.action == 'crop'):
            return True

        if(parser.left + parser.right >= 100):
            message = 'Sum of value of left and right crop arguments should be from 0 to 99'
            Colour.print(message,Colour.RED)
            sys.exit(0)
        if(parser.top + parser.bottom >= 100):
            message = 'Sum of value of top and bottom crop arguments should be from 0 to 99'
            Colour.print(message,Colour.RED)
            sys.exit(0)


    def _is_valid_percent(parser, arg):
        try:
            arg = int(arg)
        except:
            parser.error("Percentage value should be an Integer value")
        if arg < 0 or arg > 100:
            parser.error("Percentage value should be in 0 to 100")
        else:
            return arg


    def _is_valid_file(parser, arg):
        if not os.path.exists(arg):
            parser.error("The file %s does not exist!" % arg)
        else:
            return arg


    def get_parser():
        class HelpFormatter(argparse.ArgumentDefaultsHelpFormatter, argparse.RawTextHelpFormatter):
            pass

        # parser = argparse.ArgumentParser(formatter_class=HelpFormatter)
        parser = argparse.ArgumentParser()

        parser.add_argument("-v","--version",
                            action='store_true',
                            help='Display version number')

        subparsers = parser.add_subparsers(dest='action')

        trim_parser = subparsers.add_parser('trim')
        trim_parser.add_argument("inp",nargs='?',
                                type=lambda x: Args._is_valid_file(trim_parser,x),
                                help="input video file ex: input.mp4")
        trim_parser.add_argument("-s", "--start",
                                help="start time for cuting in format hh:mm:ss or mm:ss")
        time_group = trim_parser.add_mutually_exclusive_group()
        time_group.add_argument("-e", "--end",
                                help="end time for cuting in format hh:mm:ss or mm:ss")
        time_group.add_argument("-t", "--time",
                                help="clip duration in format hh:mm:ss or mm:ss")
        trim_parser.add_argument("-o", "--output",
                                help="output file name, ex: output.mp4")


        crop_parser = subparsers.add_parser('crop',formatter_class=HelpFormatter)
        crop_parser.add_argument("inp",nargs='?',
                                type=lambda x: Args._is_valid_file(crop_parser,x),
                                help="input video file ex: input.mp4")
        crop_parser.add_argument("-t", "--top",
                                default=0,
                                type=lambda x: Args._is_valid_percent(crop_parser,x),
                                help="percentage of screen to be chopped from top")
        crop_parser.add_argument("-b", "--bottom",
                                default=0,
                                type=lambda x: Args._is_valid_percent(crop_parser,x),
                                help="percentage of screen to be chopped from bottom")
        crop_parser.add_argument("-l", "--left",
                                default=0,
                                type=lambda x: Args._is_valid_percent(crop_parser,x),
                                help="percentage of screen to be chopped from left")
        crop_parser.add_argument("-r", "--right",
                                default=0,
                                type=lambda x: Args._is_valid_percent(crop_parser,x),
                                help="percentage of screen to be chopped from right")
        crop_parser.add_argument("-o", "--output",
                                help="output file name, ex: output.mp4")


        resize_parser = subparsers.add_parser('resize')
        resize_parser.add_argument("inp",nargs='?',
                                    type=lambda x: Args._is_valid_file(trim_parser,x),
                                    help="input video file ex: input.mp4")
        resize_parser.add_argument("-q", "--quality",
                                    type=int,default=50,
                                    help="output video quality (on scale of 100) (default: 50)")
        resize_parser.add_argument("-o", "--output",
                                    help="output file name, ex: output.mp4")


        extract_parser = subparsers.add_parser('extract')
        extract_parser.add_argument("inp",nargs='?',
                                    type=lambda x: Args._is_valid_file(trim_parser,x),
                                    help="input video file ex: input.mp4")
        extract_group = extract_parser.add_mutually_exclusive_group(required=True)
        extract_group.add_argument("-v","--video",
                                    action='store_true')
        extract_group.add_argument("-a","--audio",
                                    action='store_true')
        extract_parser.add_argument("-o", "--output",
                                    help="output file name")


        autocomplete(parser)
        parsed_args = parser.parse_args()
        if(not parsed_args.version and not parsed_args.action):
            Colour.print('requires atleast one command line argument',Colour.RED)
            parser.print_help(sys.stderr)
            sys.exit(1)
        return parsed_args



import os
import subprocess as sp
import sys

from .args import Args
from .meditor import Meditor
from .srbColour import Colour
from .time import Time

class Util:
    def get_filters(parser):
        filters = " "
        if(not parser.action == 'crop'):
            return filters

        Args.validate_crop_args(parser)

        crop_args = {}

        crop_args['x_point'] = str(parser.left/100)
        crop_args['y_point'] = str(parser.top/100)

        crop_args['width'] = str(1- (parser.left/100) - (parser.right/100))
        crop_args['height'] = str(1 - (parser.top/100) - (parser.bottom/100))

        filters = ' -filter:v "crop=in_w*'+crop_args['width']+':in_h*'+crop_args['height']+ \
            ':in_w*'+crop_args['x_point']+':in_h*'+crop_args['y_point']+'" '

        return filters

    def get_io(parser):
        if(not parser.inp):
            inp = input('Please enter input video file path : ')
            if(not os.path.exists(inp)):
                Colour.print('File diesnot exist',Colour.RED)
                sys.exit(0)
        else:
            inp = parser.inp

        if(not parser.output):
            if(parser.action == 'extract' and parser.audio):
                out = ''.join(inp.split('.')[:-1]) + '_output.mp3'
            else:
                out = ''.join(inp.split('.')[:-1]) + '_output.' + inp.split('.')[-1]
        else:
            out = parser.output

        if(os.path.exists(out)):
            Colour.print('[Warning] '+out+' file already exists',Colour.YELLOW)

        if(parser.action == 'extract'):
            out_ext = out.split('.')[-1]
            if(parser.video and out_ext == 'mp3'):
                Colour.print('[Warning] video output file with .mp3 extension',Colour.YELLOW)
                sys.exit(0)
            if(parser.audio and out_ext == 'mp4'):
                Colour.print('[Warning] audio output file with .mp4 extension',Colour.YELLOW)
                sys.exit(0)

        if(inp.split('.')[-1] == 'mp3' and out.split('.')[-1] == 'mp4'):
            Colour.print('[Warning] input file audio output file video',Colour.YELLOW)
            sys.exit(0)

        inp = '"' + inp + '"'
        out = '"' + out + '"'
        return inp,out

    def get_trimmer(parser,inp):
        trimmer = ' '
        if(not parser.action == 'trim'):
            return trimmer

        if(not parser.start):
            st = input('Please enter start-time in format hh:mm:ss or mm:ss : (default 00:00:00) : ')
            st.strip()
            if(st == ''):
                st = '00:00:00'
        else:
            st = parser.start
        st = Time.get_time(st)
        if(not st):
            Colour.print('Wrong format for start-time',Colour.RED)
            sys.exit(0)

        if(parser.time):
            t = parser.time
        else:
            if(parser.end):
                et = parser.end
            else:
                len_file = Meditor.getLength(inp)
                et = input('Please enter end-time in format hh:mm:ss or mm:ss : (default '+len_file+') : ')
                et.strip()
                if(et == ''): et = len_file
            et = Time.get_time(et)
            if(not et):
                Colour.print('Wrong format for end-time',Colour.RED)
                sys.exit(0)
            t = Time.get_relative_time(st,et)
            if(not t):
                Colour.print('end-time should be greater than start-time',Colour.RED)
                sys.exit(0)

        t = Time.get_time(t)
        if(not t):
            Colour.print('Wrong format for time duration',Colour.RED)
            sys.exit(0)

        trimmer = " -ss " +str(st)+ " -t " +str(t)
        return trimmer

    def get_resizer(parser):
        resizer = ' '
        if(not parser.action == 'resize'):
            return resizer

        if(parser.quality > 150):
            Colour.print('maximum value of quality is 150',Colour.RED)
            sys.exit(0)

        if(parser.quality < 0):
            Colour.print('quality cant be negative',Colour.RED)
            sys.exit(0)

        qual = parser.quality/100
        qual = 51 - qual*33
        '''
        actual crf (libx264) scale is from 0 to 51. normally 0 - 18 increase size so they are useless
        we will choose range from 18 to 51
        '''
        resizer = ' -crf ' + str(qual) + ' '
        return resizer



import os
import subprocess

class C: # for colours
    R = '\033[91m'
    G = '\033[92m'
    Y = '\033[93m'
    E = '\033[0m'


def _get_supported_distros(dependency_map):
    supported_distros = set()
    for rules in dependency_map.values():
        for key in rules.keys():
            supported_distros.add(key)
    return supported_distros


def line_adder(filename, line):
    with open(filename, 'r+') as f:
        content = f.read()
        f.seek(0, 0)
        lines = [x.strip() for x in f.readlines()]
        if(not line in lines):
            f.seek(0, 0)
            f.write(content + '\n' + line + '\n')


def get_distro(distros):
    try:
        p = subprocess.Popen(['uname','-a'], stdout=subprocess.PIPE)
        out = p.stdout.read().decode('utf-8').lower()
        for d in distros:
            if d in out:
                return d
        return None
    except:
        return None


def is_installed(soft):
    dump_out = ' > /dev/null 2>&1'
    help_opt = ' --help '
    a = os.system(soft + help_opt + dump_out)
    if a == 0 or a == 256:
        '''
        0 means return 0
        256 means return 1, generally for those who dont have --help
        if command not found it will return 32512
        '''
        return True
    return False


def install_arg_complete():
    if is_installed('register-python-argcomplete'):
        line = 'eval "$(register-python-argcomplete medipack)"'
        filename = os.environ['HOME'] + '/.bashrc'
        line_adder(filename,line)


def install_dependencies(dependency_map, verbose = False):
    supported_distros = _get_supported_distros(dependency_map)
    distro = get_distro(supported_distros)
    if(not distro and verbose):
        print(C.R+'unrecognised distro, please contact srbcheema2@gmail.com for full support for your distro'+C.E)
    elif(distro and verbose):
        print(C.G+'Distro detected to be '+distro+' based'+C.E)

    all_installed = True

    for d in dependency_map.keys():
        if is_installed(d):
            continue
        rules = dependency_map[d]
        if distro and distro in rules.keys():
            print(C.G+'installing '+d+' dependency'+C.E)
            os.system(rules[distro])
            if not is_installed(d):
                print(C.Y+'please install ' +d+ ' dependency manually'+C.E)
                print(C.Y+'try command : '+rules[distro]+C.E)
                all_installed = False
        else:
            print(C.Y+'Please install ' +d+' dependency manually'+C.E)
            all_installed = False

    return all_installed


if __name__ == '__main__':
    dependency_map = {
        'register-python-argcomplete':{
            'ubuntu':'sudo apt install python-argcomplete',
        },
        'ffmpeg':{
            'ubuntu':'sudo apt install ffmpeg',
        },
        'figlet':{
            'ubuntu':'sudo apt install figlet',
        },
    }
    install_dependencies(dependency_map,verbose=True)
    install_arg_complete()



#!/usr/bin/env python3
import os
from setuptools import setup, find_packages

try:
    '''
    in no case, setup should fail.
    take care of it that setup.py should not fail due to change in other files.
    if it fails it will cause a serious problem
    we can detect that from version number that will be installed during failure
    if your version is 1.0.0 please report srbcheema2@gmail.com
    '''
    from medipack import __version__, __mod_name__
    from medipack.dependencies.dependencies import dependency_map
    from medipack.dependencies.dependency import install_arg_complete, install_dependencies
    install_dependencies(dependency_map,verbose=True)
    install_arg_complete()
except:
    __version__ = '1.0.0'
    __mod_name__ = 'medipack'
    print('something really bad happened, please report to srbcheema2@gmail.com')


try:
    with open("README.md", 'r') as f:
        long_description = f.read()
    with open('requirements.txt', 'r') as f:
        requirements = [line.strip() for line in f.readlines()]
except:
    long_description='A command line tool for media editing',
    requirements = []

setup(
    name=__mod_name__,
    version=__version__,
    description='A command line tool for media editing',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Sarbjit Singh',
    author_email='srbcheema1@gmail.com',
    url='http://github.com/srbcheema1/'+__mod_name__,

    # packages=[__mod_name__,__mod_name__+'.lib',__mod_name__+'.dependencies'],  #same as name of directories
    packages=find_packages(), # provides same list, looks for __init__.py file in dir
    include_package_data=True,
    install_requires=requirements, #external packages as dependencies

    entry_points={
        'console_scripts': [__mod_name__+'='+__mod_name__+'.main:main']
    },

    classifiers=[
        'Operating System :: POSIX :: Linux',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
    ],
    license='MIT License',
)
